Package: InPAS
Title: Identify Novel Alternative PolyAdenylation Sites (PAS) from
        RNA-seq data
Version: 2.21.0
Authors@R: c(person("Jianhong", "Ou", role = c("aut", "cre"), 
             email = "jou@morgridge.org"), 
             person("Haibo", "Liu", role = c("aut"), 
             email = "haibo.liu@umassmed.edu"),
             person("Lihua Julie", "Zhu", role = c("aut"), 
             email = "Julie.Zhu@umassmed.edu"), 
             person("Sungmi M.", "Park", role = "aut"),
             person("Michael R.", "Green", role = "aut"))
Maintainer: Jianhong Ou <jou@morgridge.org>
Description: Alternative polyadenylation (APA) is one of the important post-
             transcriptional regulation mechanisms which occurs in most human
             genes. InPAS facilitates the discovery of novel APA sites and the
             differential usage of APA sites from RNA-Seq data. It leverages 
             cleanUpdTSeq to fine tune identified APA sites by removing false 
             sites.
biocViews: Alternative Polyadenylation, Differential Polyadenylation
        Site Usage, RNA-seq, Gene Regulation, Transcription
License: GPL (>= 2)
Imports:
        AnnotationDbi,batchtools,Biobase,Biostrings,BSgenome,cleanUpdTSeq,
        depmixS4,dplyr,flock,future,future.apply,GenomeInfoDb,GenomicRanges,
        GenomicFeatures, ggplot2, IRanges, limma,
        magrittr,methods,parallelly, plyranges, preprocessCore,
        readr,reshape2, RSQLite, Seqinfo, stats, S4Vectors, utils
Depends: R (>= 3.5)
Suggests: BiocGenerics,BiocManager, BiocStyle,
        BSgenome.Mmusculus.UCSC.mm10, BSgenome.Hsapiens.UCSC.hg19,
        EnsDb.Hsapiens.v86, EnsDb.Mmusculus.v79, knitr, markdown,
        rmarkdown, rtracklayer, RUnit, grDevices,
        TxDb.Hsapiens.UCSC.hg19.knownGene,TxDb.Mmusculus.UCSC.mm10.knownGene
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
LazyData: true
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/InPAS
git_branch: devel
git_last_commit: 15afd99
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:19:43 UTC; biocbuild
Author: Jianhong Ou [aut, cre],
  Haibo Liu [aut],
  Lihua Julie Zhu [aut],
  Sungmi M. Park [aut],
  Michael R. Green [aut]
