# InPAS

[![platforms](http://bioconductor.org/shields/availability/devel/InPAS.svg)](http://bioconductor.org/packages/devel/bioc/html/InPAS.html)
[![build](http://bioconductor.org/shields/build/devel/bioc/InPAS.svg)](http://bioconductor.org/packages/devel/bioc/html/InPAS.html)

Identification of Novel alternative PolyAdenylation Sites (PAS)

Alternative polyadenylation (APA) is one of the important post-transcriptional regulation mechanisms which occurs in most human genes. InPAS facilitates the discovery of novel APA sites from RNAseq data. It leverages cleanUpdTSeq to fine tune identified APA sites.

## Installation

To install this package, start R and enter:

```r
library(BiocManager)
BiocManager::install("InPAS")
```

## Documentation

To view documentation of InPAS, start R and enter:
```r
browseVignettes("InPAS")
```

