% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00.set_globals.R
\name{getInPASGenome}
\alias{getInPASGenome}
\title{Get the globally defined genome}
\usage{
getInPASGenome()
}
\description{
This function will retrieve the genome that is currently in use by InPAS.
}
